// common.h				Last updated: Mar 26, 98

#ifndef COMMON_H
#define COMMON_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

typedef enum {FALSE, TRUE, FALSE_2} BOOL;

// Status of node in the multicasting tree
// UNSEEN   : not in the MC tree
// LEAF_NODE: a participant node (in the MC tree)
// INTREE   : in the MC tree, but not a participant (just an intermediate node)
// FRINGE   : used only by Dijkstra's algorithm.  See alg.cc, method ShortestPath.
typedef enum {INTREE, FRINGE, UNSEEN, LEAF_NODE} STATUS;

// Type of adjacency list of node
typedef enum {FORWARD, BACKWARD } ADJ_TYPE;

// Type of cost to find the path
typedef enum {WEIGHT, DELAY, AVG, CONGESTION} COST_TYPE;

// User request type
typedef enum {SETUP, JOIN, LEAVE, TEARDOWN } USER_REQ_TYPE;

// Data type of user defined data file, *.dat *.req
typedef enum {NOT_INFO, NODE_INFO, SESSION_INFO, GRPREQ_ID, USER_REQ_INFO, EDGE_INFO} INFO_TYPE;

// Type of action for bandwidth update in the edge
// EDGE_ADD: join request
// EDGE_DEC: leave request
typedef enum { EDGE_ADD, EDGE_DEC } ACTION_TYPE;

// Type of algorithm
typedef enum { PROPOSED_ALG, SHORTEST_PATH } ALG_TYPE;

// Type of path
typedef enum { SHARED_PATH, NEW_PATH } PATH_TYPE;

// This structure is used by Algorithm and Path class to store the weight and delay
typedef struct COST {
	int	weight;
	int 	delay;
        int 	hopCount;

	COST():weight(0), delay(0), hopCount(0) {}
	COST(int w, int d, int h) { weight = w; delay = d; hopCount = h; }
	friend COST operator+(COST a, COST b);
};
inline COST operator+(COST a, COST b) {
	return (COST(a.weight+b.weight, a.delay+b.delay, a.hopCount+b.hopCount));
}

// Constants
#define UN_LIMIT   	-1
#define	NL		'\n'		// new line
#define NIL	 	0	  	// nil pointer
#define INFINITY 	1.0E+36  	// very very big number (FLT_MAX = 1.0E+37)
#define MAXINT		+32767		// (INT_MAX = +32767)
#define SYS_PARMS_FILE	"sysParms.dat"	// input file containing system parameters

// Macros
#define maxValue(a,b)	((a) > (b) ? (a) : (b))

// Conditional compilation flags
//#define DEBUG_LEVEL_1		// print out actions of each join/leave
//#define DEBUG_LEVEL_2		// print out MC trees and tree paths
//#define DEBUG_LEVEL_3		// print out all costs in details
//#define FUNCTION_TRACE

#endif


////////////////////////////////////////////////////////////////////////////////////
//
// Updates:
// Feb 12: - add CONGESTION to COST_TYPE
//	   - modify struct COST 
//
////////////////////////////////////////////////////////////////////////////////////