#ifndef FileIO_H
#define FileIO_H

#include "common.h"
#include "manager.h"

// edge_data : edge from source_node to target_node
// target_node, weight, delay, bandwidth
// If the value of bandwidth is "-1", it is unlimit
#define	DATA_IN_EDGE	4

// S : Session id : Source Node
#define	DATA_IN_SESSION		2

// User Request => 
// U : Request_node_ID : Session_ID : Req_type : band_width : max_delay : max_hops
//  Req_type : SETUP(0), JOIN(1), LEAVE(2), TEARDOWN(3)
#define	DATA_IN_USER_REQ	6

///////////////////////////////////////////////////////////////////////////////
//
//	Structure Name : READ_ITEM
//
//
///////////////////////////////////////////////////////////////////////////////
typedef struct READ_ITEM {
	int	d1;
	int 	d2;
	int 	d3;
	int 	d4;
	int 	d5;
	int 	d6;

	READ_ITEM():d1(-1),d2(-1),d3(-1),d4(-1),d5(-1),d6(-1) {}
	void	Init() {
		d1=-1; d2=-1; d3=-1; d4=-1; d5=-1; d6=-1;
	};

	Display() {
		printf("d1[%d], d2[%d], d3[%d], d4[%d], d5[%d], d6[%d]\n", 
			d1, d2, d3, d4, d5, d6);
	}
};

///////////////////////////////////////////////////////////////////////////////
//
//	Class Name : FileIO
//
//	Description : 
//		Class to do file I/O related jobs.
//		This class has a pointer to a data_manager object.
//
///////////////////////////////////////////////////////////////////////////////
class FileIO {
private:
	char	*net_dat_file;  // file name, containing network graph
	char	*grp_req_file;  // file name, containing sessions and requests
	DataManager*	data_manager;

public:
	FileIO(DataManager* data, char* dat_file, char* req_file);
	~FileIO();

	void	SetDatFile(char* dat_file);
	char*	GetDatFile(void) const { return net_dat_file; }

	void	SetReqFile(char* req_file);
	char*	GetReqFile(void) const { return grp_req_file; }

	DataManager*	GetDataMgr(void) const { return data_manager; }
	
	BOOL	FileRead(char* dat_file, char* req_file);

	// Commonly used methods
        INFO_TYPE	GetInfoType(char ch);
        int     IsDigit(char ch); 
        int     ReadItem(char* sbuf, int num_of_item, int& cur_idx, READ_ITEM& data);

	// Methods to read graph information
	BOOL	ReadGraphInfo(char* dat_file);

	// Methods to read Session and group request information from file
	BOOL	ReadSess_MC(char* req_file);
	BOOL    ReadSessionInfo(FILE* fp);
	BOOL    ReadGroupInfo(FILE* fp);
	GrpReq*	ReadGroupReqID(char* sbuf);
	BOOL    ReadUserReq(GrpReq* mc_grp, char* sbuf);

	BOOL 	IsEndOfData(char* sbuf);
	BOOL   	IsSessionInfo(char* sbuf);
	BOOL   	IsReqInfo(char* sbuf);
	BOOL	ReadOneSession(char* sbuf);
	BOOL	ReadSystemParms();
};
#endif
