///////////////////////////////////////////////////////////////////////////////
//
//	File name : grp_req.cc
//
//	Description :
//		This file implements methods of classes
//		GrpReq, UserReq and QoS.
//
//	Authors : Jin-Hwee Park / U.T.
// 	Date 	: Feb-01-1998
//
///////////////////////////////////////////////////////////////////////////////

#include "grp_req.h"

//
// Display information of a user request (printed on screen)
//
void	UserReq::Display() 
{
	printf("* REQUEST Node[%d] : session[%d], type[%d], delay[%d], band[%d], max_hops[%d]\n",
		node_id, sess_id, req_type, qos.GetDelay(), qos.GetBandwidth(), 
		qos.GetHop());
}

//
// Display information of a user request (printed to a file)
//
void	UserReq::Display(FILE* fp) 
{
	fprintf(fp, "* REQUEST Node[%d] : session[%d], type[%d], delay[%d], band[%d], max_hops[%d]\n",
		node_id, sess_id, req_type, qos.GetDelay(), qos.GetBandwidth(), 
		qos.GetHop());
}

//
// Delete the list of single requests
//
GrpReq::~GrpReq() 
{
	UserReq* tmp;

	while(first_req != NIL) {
		tmp = first_req;
		first_req = first_req->GetNext();  // first_req->next
		delete tmp;
	}
}

//
// Add "user_req" to the end of the list of requests of this group.
// Return TRUE.
//
BOOL 	GrpReq::AddUserReq(UserReq *user_req)
{
#ifdef FUNCTION_TRACE
  fprintf(stderr, ".......... AddUserReq() \n");
#endif

	UserReq *tmp0, *tmp1;
	int user_node = user_req->GetNodeID();
	int user_sess = user_req->GetSessionID();

	// IncUserNum();   // increment total number of requests
			   // should we increment if the request is duplicated?
	tmp1 = first_req;
	if (tmp1 == NIL) {
		first_req = user_req;  // first and only request
		return TRUE;
	}

	while(tmp1 != NIL) {  // traverse to the end of the list of requests
		if (tmp1->GetNodeID() == user_node &&
		    tmp1->GetSessionID() == user_sess &&
		    tmp1->GetUserReq() == user_req->GetUserReq())
			{
			printf("Duplicate request (consider it anyway)!\n");
			user_req->Display();
			// return FALSE;    // duplicate request => ignore it
			}

		tmp0 = tmp1;
		tmp1 = tmp1->GetNext();  // tmp1 = tmp1->next
	}
	tmp0->SetNext(user_req);  // tmp0->next = user_req;
	IncUserNum();   	  // increment total number of requests
	return TRUE;
}

//
// Display the list of requests in this group
//
void	GrpReq::Display()
{
	UserReq* tmp;

	printf("*** Request Group ===> Group_ID[%d], Number_Of_Requests[%d]\n", 
					grp_id, user_num+1);
	tmp = first_req;
	while(tmp != NIL) {
		tmp->Display();
		tmp= tmp->GetNext();  // tmp= tmp->next;
	}
	printf("\n");
}
