#ifndef GrpReq_H
#define GrpReq_H

#include "common.h"

////////////////////////////////////////////////////////////////////////////////
//
//	Class Name : Qos
//
//	Description :
//		QOS for each user request
//
////////////////////////////////////////////////////////////////////////////////
class Qos {
private:
	int	qos_type;
	int	bandwidth;
	int	e2e_delay;
	int	max_hops;

public:
	Qos() {
		qos_type=-1;
		bandwidth=-1;
		e2e_delay=-1;
		max_hops=-1;
	};
	Qos(int b, int d, int h):bandwidth(b), e2e_delay(d), max_hops(h) {};

	int	GetType(void) const { return qos_type; }
	void	SetType(int t) { qos_type = t; }
	
	int	GetBandwidth(void) const { return bandwidth; }
	void	SetBandwidth(int b) { bandwidth = b; }
	
	int	GetDelay(void) const { return e2e_delay; }
	void	SetDelay(int d) { e2e_delay = d; }
	
	int	GetHop(void) const { return max_hops; }
	void	SetHop(int h) { max_hops = h; }
};

////////////////////////////////////////////////////////////////////////////////
//
//	Class Name : UserReq
//
//	Description : 
//		User request.
//		Each request has a session ID, requesting node ID and Qos.
//		Requests of the same request group are linked into a list.
//
////////////////////////////////////////////////////////////////////////////////
class UserReq {
private:
	int	node_id;	// User request node ID
	int	sess_id;	// request session ID
	USER_REQ_TYPE	req_type;	// User request types
				// 0. set up
				// 1. join
				// 2. leave
				// 3. tear down
	Qos	qos;		// Requested QoS;
	UserReq *next;		// Point to next request in linked list of requests

public:
	UserReq(int n, int s, USER_REQ_TYPE type, Qos q) {
		node_id=n;
		sess_id=s;
		req_type=type;
		qos = q;
		next = NIL;
	}

	int	GetNodeID() { return node_id; }  // return requesting node ID
	int	GetSessionID() { return sess_id; }  // return session ID

	USER_REQ_TYPE GetUserReq(void) const { return req_type; }
	void	SetUserReq(USER_REQ_TYPE u) { req_type = u; }

	Qos	GetQos(void) const { return qos; }  // return requested QoS

	UserReq* GetNext() { return next; }  // get next request of the list
	int	SetNext(UserReq* n_user) { next = n_user; }
	void	Display();
	void	Display(FILE* fp);
};

////////////////////////////////////////////////////////////////////////////////
//
//	Class Name : GrpReq
//
//	Description : 
//		A group of user requests (considered to be
//		submitted at the same time).
//
//	Note:
//	- Requests of the same group are linked into a list.
//	- Each request group is uniquely identified by a group ID (grp_id).
//	- Many request groups are linked into a list of request groups.
//
//	Note: The correct name for this class is RequestGroup
//
////////////////////////////////////////////////////////////////////////////////
class GrpReq {

private:
	int	grp_id;		// group ID
	int	user_num;	// Total number of requests in the group
	UserReq *first_req;	// Pointer to head of list of single requests
	GrpReq* next;		// Pointer to next request group

public:
      	GrpReq(int id) { 
		grp_id = id;
		user_num = 0;
		first_req = NIL;
		next = NIL;
	};
	~GrpReq();

	int	GetGrpID(void) const { return grp_id; }
	void	SetGrpID(int id) { grp_id = id; }
	
	int	GetUserNum(void) const { return user_num; }  
		// return number of requests
		// not used for the time being
	void	IncUserNum(void) { user_num++; }  // increment number of requests

	UserReq*	GetUserReqHead(void) const { return first_req; }
	BOOL	AddUserReq(UserReq *user_req);

	GrpReq	*GetNext() { return(next); };  // get next request group
	void	SetNext(GrpReq *new_mc) { next = new_mc; }

	void		Display();
};

#endif
