// manager.h				Last updated: Feb 24, 98

#ifndef DataManager_H
#define DataManager_H

#include "common.h"
#include "graph.h"
#include "grp_req.h"
#include "session.h"


///////////////////////////////////////////////////////////////////////////////
//
//	Class Name : NEW_ENV
//
//	Description : 
//		Class to contain the network control information
//
//	Note : Not used for the time being
//
///////////////////////////////////////////////////////////////////////////////
class NET_ENV {
public:
	int	assym_ratio;	// Asymmetric of graph	ratio of opposite edges
	int	max_stream;	// Max stream
	int	num_part;	// NUmber of participant in a session
	int	loop_type;
	int	loop_start;
	int	loop_end;
	int	loop_step;
};

///////////////////////////////////////////////////////////////////////////////
//
//	Class Name : Parameters
//
///////////////////////////////////////////////////////////////////////////////
class Parameters
{
private:
  float delayCostFactor;
  float sharedCostFactor;
  float addedCostFactor;
  float delayEpsilon;

public:
  Parameters()
  { delayCostFactor  	= 0.0;
    sharedCostFactor 	= 0.0;
    addedCostFactor  	= 0.0;
    delayEpsilon 	= 0.1;
  };

  Parameters(float delayFact, float sharedFact, float addedFact, float delayEps)
  { delayCostFactor 	= delayFact;
    sharedCostFactor 	= sharedFact;
    addedCostFactor 	= addedFact;
    delayEpsilon 	= delayEps;
  };

  ~Parameters() {return;};

  float GetDelayCostFactor() 	const {return delayCostFactor;}
  float GetSharedCostFactor() 	const {return sharedCostFactor;}
  float GetAddedCostFactor() 	const {return addedCostFactor;}
  float GetDelayEpsilon() 	const {return delayEpsilon;}

  void  SetDelayCostFactor(float dataFact) 	{delayCostFactor  = dataFact;}
  void  SetSharedCostFactor(float dataFact) 	{sharedCostFactor = dataFact;}
  void  SetAddedCostFactor(float dataFact) 	{addedCostFactor  = dataFact;}
  void  SetDelayEpsilon(float dataFact) 	{delayEpsilon     = dataFact;}

  void  Display()
  { printf("Delay cost factor  = %.4f \n", delayCostFactor);
    printf("Shared cost factor = %.4f \n", sharedCostFactor);
    printf("Added cost factor  = %.4f \n", addedCostFactor);
    printf("Delay epsilon      = %.4f \n", delayEpsilon);
  }
};

///////////////////////////////////////////////////////////////////////////////
//
//	Class Name : DataManager
//
//	Description : 
//		Class holding data structures used in the program:
//		- network graph
//		- list of sessions
//		- list of request groups, each group being a list of requests
//
///////////////////////////////////////////////////////////////////////////////
class DataManager {
private:
	NET_ENV*	net_env;
	Graph*	graph;		// Network graph
	GrpReq*	grp_req_head;	// List of request groups
	int	grp_req_num;	// Number of request groups
	Session	*session_head;	// List of sessions
	int	sess_num;	// Number of sessions
	Parameters* parameters; // Parameters

public:
	DataManager() {
		net_env = NIL;
		graph = NIL;
		grp_req_head = NIL;
		grp_req_num = 0;
		session_head = NIL;
		sess_num = 0;
		parameters = NIL;
	};

	~DataManager();

	// Parameters
	Parameters* GetParameters() const {return parameters;}
	void	    SetParameters(Parameters* parms) {parameters = parms;}

	// Network environment related methods
	NET_ENV*	GetNetEnv(void) const { return net_env; }
	void	SetNetEnv(NET_ENV* n) { net_env = n; }
	
	// Graph related methods
	Graph*	GetGraph() { return graph; }
	void	SetGraph(Graph* gr) { graph = gr; }


	// Request related methods
	GrpReq*	GetGrpReqHead(void) const { return grp_req_head; }
	int	GetGrpReqNum(void) const { return grp_req_num; }
	int	AddGrpReq(GrpReq* grp_req);
	void 	RemoveGrpReq(int grp_req_id);

	// Session related methods
	Session*	GetSessionHead(void) const { return session_head; }
	int	GetSessionNum(void) const { return sess_num; }
	int	AddSession(Session*	sess);
	void 	RemoveSession(int sess_id);
	Session*	FindSession(int session_id);

	// Utilities
	void	DeleteDynamicData();
	void	Display();
	void 	AllReq_Display();
	void 	AllSess_Display();
	void    AllMCTree_Display();
	void	AllPaths_Display();
};

#endif


///////////////////////////////////////////////////////////////////////////////
//
// Updates:
// Feb 24: added class Parameters (UT)
//
///////////////////////////////////////////////////////////////////////////////